using System.Text.Json;
using System.Text.Json.Serialization;

namespace KanbnToQuartz.Models;

public record AppSettings(string Company,
                          string Destination,
                          string DirectoryFindReplace,
                          string DirectoryBFindReplace,
                          string FileFindReplace,
                          string Source,
                          string TextBFindReplace,
                          string TextFindReplace,
                          string TextSearchExtension,
                          string WorkingDirectoryName)
{

    public override string ToString() {
        string result = JsonSerializer.Serialize(this, AppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    public static AppSettings Get(IConfigurationRoot configurationRoot) {
        AppSettings? result;
        try {
#pragma warning disable IL3050, IL2026
            result = configurationRoot.Get<AppSettings>() ?? throw new Exception();
#pragma warning restore IL3050, IL2026
        } catch (Exception) {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers) {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }

            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }

        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class AppSettingsSourceGenerationContext : JsonSerializerContext
{
}