using KanbnToQuartz.Models;
using System.Collections.ObjectModel;

namespace KanbnToQuartz.Services;

public class CopyService
{

    private readonly AppSettings _AppSettings;

    public CopyService(AppSettings appSettings) =>
        _AppSettings = appSettings;

    private record Record(string Source, string Directory, string Destination, string? Text);

    private static ReadOnlyCollection<Record> GetRecords(AppSettings appSettings)
    {
        List<Record> results = [];
        string path;
        string fileB;
        string pathB;
        string pathC;
        string pathD;
        string textB;
        string? text;
        Record record;
        string before;
        string[] lines;
        string pathName;
        string directory;
        string[] segments;
        string destination;
        string relativePath;
        string directoryName;
        string directoryNameB;
        string relativeDirectory;
        string textBFindReplace = appSettings.TextBFindReplace;
        string sourceDirectory = Path.GetFullPath(appSettings.Source);
        string[] fileFindReplace = appSettings.FileFindReplace.Split('|');
        string[] textFindReplace = appSettings.TextFindReplace.Split('|');
        string sourceDirectoryName = Path.GetDirectoryName(sourceDirectory);
        string destinationDirectory = Path.GetFullPath(appSettings.Destination);
        string[] directoryFindReplace = appSettings.DirectoryFindReplace.Split('|');
        string[] directoryBFindReplace = appSettings.DirectoryBFindReplace.Split('|');
        string[] files = Directory.GetFiles(sourceDirectory, "*", SearchOption.AllDirectories);
        int sourceDirectoryLength = sourceDirectory.Length;
        foreach (string file in files)
        {
            if (sourceDirectory.Contains(directoryFindReplace[0]))
                continue;
            fileB = file.Replace(directoryFindReplace[0], directoryFindReplace[1]);
            directoryName = Path.GetDirectoryName(fileB);
            if (!file.EndsWith(appSettings.TextSearchExtension))
                text = null;
            else
            {
                lines = File.ReadAllLines(file);
                relativeDirectory = Path.GetRelativePath(sourceDirectoryName, directoryName);
                for (int i = 0; i < lines.Length; i++)
                {
                    if (!lines[i].Contains(textBFindReplace))
                        continue;
                    if (!lines[i].EndsWith(')') && !lines[i].EndsWith(")~~"))
                        continue;
                    if (lines[i].StartsWith(textBFindReplace))
                        path = lines[i].Substring(textBFindReplace.Length, lines[i].Length - 1 - textBFindReplace.Length);
                    else
                    {
                        segments = lines[i].Split("~~");
                        if (segments.Length < 2 || !segments[1].StartsWith(textBFindReplace))
                            continue;
                        path = segments[1].Substring(textBFindReplace.Length, segments[1].Length - 1 - textBFindReplace.Length);
                    }
                    if (string.IsNullOrEmpty(directoryBFindReplace[0]))
                    {
                        pathD = path;
                        pathName = Path.GetFileName(path);
                    }
                    else
                    {
                        pathB = path.Replace(directoryBFindReplace[0], directoryBFindReplace[1]);
                        pathC = directoryBFindReplace[1][0] != '/' ? pathB : pathB.Replace('\\', '/');
                        pathName = Path.GetFileName(pathC);
                        pathD = $"{pathC}{directoryFindReplace[1]}{pathName}";
                    }
                    relativePath = Path.GetRelativePath(sourceDirectoryName, pathD);
                    lines[i] = lines[i].Replace($"{textBFindReplace}{path}", $"- [{pathName}]({relativePath}");
                }
                textB = string.Join(Environment.NewLine, lines);
                text = textB.Replace(textFindReplace[0], textFindReplace[1].Replace("%", relativeDirectory));
            }
            before = $"{destinationDirectory}{fileB[sourceDirectoryLength..]}";
            directoryNameB = Path.GetFileName(Path.GetDirectoryName(directoryName));
            destination = before.Replace(fileFindReplace[0], fileFindReplace[1].Replace("%", directoryNameB));
            directory = Path.GetDirectoryName(destination) ?? throw new Exception();
            record = new(Source: file, Directory: directory, Destination: destination, Text: text);
            results.Add(record);
        }
        return results.AsReadOnly();
    }

    private static void CreateDirectories(ReadOnlyCollection<Record> records)
    {
        List<string> collection = [];
        foreach (Record record in records)
        {
            if (collection.Contains(record.Directory))
                continue;
            collection.Add(record.Directory);
            if (!Directory.Exists(record.Directory))
                _ = Directory.CreateDirectory(record.Directory);
        }
    }

    public void CopyDirectory()
    {
        if (!Directory.Exists(_AppSettings.Source))
            _ = Directory.CreateDirectory(_AppSettings.Source);
        if (Directory.Exists(_AppSettings.Destination))
            Directory.Delete(_AppSettings.Destination, recursive: true);
        ReadOnlyCollection<Record> records = GetRecords(_AppSettings);
        CreateDirectories(records);
        foreach (Record record in records)
        {
            if (string.IsNullOrEmpty(record.Text))
                File.Copy(record.Source, record.Destination);
            else
                File.WriteAllText(record.Destination, record.Text);
        }
    }
}